///////////////////////////////////////////////////////////
//  Angle.h
//  Implementation of the Interface Angle
//  Created on:      15-Mar-2012 09:39:14
///////////////////////////////////////////////////////////

#if !defined(EA_84BDA61B_7E59_4fd9_BF11_B3F65259DB63__INCLUDED_)
#define EA_84BDA61B_7E59_4fd9_BF11_B3F65259DB63__INCLUDED_

#include <string>
#include <memory>
#include <vector>
namespace org
{
	namespace omg
	{
		namespace tacsit
		{
			namespace geometry
			{
				/**
				 * The angular distance between two intersecting lines.
				 * <p>
				 * {@linkplain Angle} objects are immutable.  All methods that operate on the
				 * state of the {@linkplain Angle} do not modify its value.
				 */
				class Angle
				{

				public:
					/**
					 * Returns a new {@linkplain Angle} which represents the sum of this
					 * angular distance and the parameter (this + that).
					 * <p>
					 * Neither this {@linkplain Angle} nor the parameter are modified.
					 * @param angle The {@linkplain Angle} to add to this {@linkplain Angle}.
					 * @return A new {@linkplain Angle} which represents (this + that).
					 */
					virtual std::auto_ptr<Angle> add(std::auto_ptr<Angle> angle) =0;
					/**
					 * Gets the value of this {@linkplain Angle} as expressed in degrees.
					 * 
					 * @return The degree value angular measurement.
					 */
					virtual double getDegrees() =0;
					/**
					 * Gets the value of this {@linkplain Angle}, as expressed in radians.
					 * @return The radian value angular measurement.
					 */
					virtual double getRadians() =0;
					/**
					 * Returns a new {@linkplain Angle} which represents the difference of this
					 * angular distance and the parameter (this - that).
					 * <p>
					 * Neither this angle nor the parameter are modified.
					 * @param angle The {@linkplain Angle} to subtract from this {@linkplain Angle}.
					 * @return A new {@linkplain Angle} which represents (this - that).
					 */
					virtual std::auto_ptr<Angle> subtract(std::auto_ptr<Angle> angle) =0;

				};

			}

		}

	}

}
#else
namespace org
{
	namespace omg
	{
		namespace tacsit
		{
			namespace geometry
			{
				class Angle;
			}
		}
	}
}
#endif // !defined(EA_84BDA61B_7E59_4fd9_BF11_B3F65259DB63__INCLUDED_)
