///////////////////////////////////////////////////////////
//  Distance.h
//  Implementation of the Interface Distance
//  Created on:      15-Mar-2012 09:39:15
///////////////////////////////////////////////////////////

#if !defined(EA_754BA5DA_AB14_40d8_B8A9_9947BFD4D3C6__INCLUDED_)
#define EA_754BA5DA_AB14_40d8_B8A9_9947BFD4D3C6__INCLUDED_

#include <string>
#include <memory>
#include <vector>
namespace org
{
	namespace omg
	{
		namespace tacsit
		{
			namespace geometry
			{
				/**
				 * A representation of the space between two points.
				 * <p>
				 * {@linkplain Distance} objects are immutable.  All methods that operate on the
				 * state of the {@linkplain Distance} do not modify its value.
				 */
				class Distance
				{

				public:
					/**
					 * Creates a new {@linkplain Distance} which represents the sum of
					 * this {@linkplain Distance} and the parameter (this + that).
					 * <p>
					 * Neither this distance nor the parameter are modified.
					 * @param distance The {@linkplain Distance} to add.
					 * @return A new {@linkplain Distance} which represents (this + that).
					 */
					virtual std::auto_ptr<Distance> add(std::auto_ptr<Distance> distance) =0;
					/**
					 * Creates a new {@linkplain Distance} which represents this
					 * {@linkplain Distance} divided by a scalar value (this / scalarValue).
					 * This {@linkplain Distance} is not modified.
					 * <p>
					 * A typical usage of this would be in a formula that requires a {@linkplain
					 * Distance};
					 * for instance, the computing half the distance distance traveled.  In this case,
					 * the
					 * total distance is a {@linkplain Distance}, the scalar value being divided is 2,
					 * and
					 * the return value is the half the total distance.
					 * @param scalarValue The scalar value to divide this {@linkplain Distance} by.
					 * @return A new {@linkplain Distance} which represents (this / scalarValue).
					 */
					virtual std::auto_ptr<Distance> dividedBy(double scalarValue) =0;
					/**
					 * Creates a new {@linkplain Distance} that represents this
					 * {@linkplain Distance} by another distance (this / that).
					 * This {@linkplain Distance} is not modified.
					 * <p>
					 * Note: When dividing a unit of measurement by the same unit of measurement, the
					 * units cancel each other out, and the resulting value is scalar.
					 * @param distance The {@linkplain Distance} to divide this {@linkplain Distance}
					 * by.
					 * @return A scalar value which represents (this / that).
					 */
					virtual double dividedBy(std::auto_ptr<Distance> distance) =0;
					/**
					 * Gets the value of this {@linkplain Distance}, as expressed in feet.
					 * @return The distance in feet.
					 */
					virtual double getFeet() =0;
					/**
					 * Gets the value of this {@linkplain Distance}, as expressed in kilometers.
					 * @return The distance in kilometers.
					 */
					virtual double getKilometers() =0;
					/**
					 * Gets the value of this {@linkplain Distance}, as expressed in meters.
					 * @return The distance in meters.
					 */
					virtual double getMeters() =0;
					/**
					 * Gets the value of this {@linkplain Distance}, as expressed in miles.
					 * @return The distance in miles.
					 */
					virtual double getMiles() =0;
					/**
					 * Gets the value of this {@linkplain Distance}, as expressed in nautical miles.
					 * @return The distance in nautical miles.
					 */
					virtual double getNauticalMiles() =0;
					/**
					 * Gets the value of this {@linkplain Distance}, as expressed in yards.
					 * @return The distance in yards.
					 */
					virtual double getYards() =0;
					/**
					 * Creates a new {@linkplain Distance} which represents this
					 * {@linkplain Distance} times a scalar value (this * scalarValue).
					 * This {@linkplain Distance} is not modified.
					 * <p>
					 * A typical usage of this would be in a formula that requires a {@linkplain
					 * Distance};
					 * for instance, the circumference of a circle = 2 * Pi * radius.  In this case,
					 * the
					 * radius is a {@linkplain Distance}, the scalar value being multiplied is (2 *
					 * Pi),
					 * and the return value is the circumference.
					 * <p>
					 * Note: No option is given to multiplyBy({@linkplain Distance}), because the
					 * logical
					 * result of such a function would be an Area value, not a {@linkplain Distance}.
					 * @param scalarValue The scalar value to multiply this {@linkplain Distance} by.
					 * @return A new {@linkplain Distance} which represents (this * scalarValue).
					 */
					virtual std::auto_ptr<Distance> multipliedBy(double scalarValue) =0;
					/**
					 * Creates a new distance which represents the difference of this
					 * {@linkplain Distance} and the parameter (this - that).
					 * <p>
					 * Neither this {@linkplain Distance} nor the parameter are modified.
					 * @param distance The {@linkplain Distance} to subtract.
					 * @return A new {@linkplain Distance} which represents (this - that).
					 */
					virtual std::auto_ptr<Distance> subtract(std::auto_ptr<Distance> distance) =0;

				};

			}

		}

	}

}
#else
namespace org
{
	namespace omg
	{
		namespace tacsit
		{
			namespace geometry
			{
				class Distance;
			}
		}
	}
}
#endif // !defined(EA_754BA5DA_AB14_40d8_B8A9_9947BFD4D3C6__INCLUDED_)
