///////////////////////////////////////////////////////////
//  Entity.h
//  Implementation of the Interface Entity
//  Created on:      15-Mar-2012 09:41:27
///////////////////////////////////////////////////////////

#if !defined(EA_4AE38760_7BAE_4a2f_B9C7_AFCBB6C8A313__INCLUDED_)
#define EA_4AE38760_7BAE_4a2f_B9C7_AFCBB6C8A313__INCLUDED_

#include "GeodeticPosition.h"
#include "EntityType.h"

#include <string>
#include <memory>
#include <vector>
namespace org
{
	namespace omg
	{
		namespace tacsit
		{
			namespace controller
			{
				/**
				 * The Entity class is an abstraction of the tactical objects that can be
				 * displayed on a Tactical Situation display (TacSit).
				 * 
				 * Entities are modeled such that it is possible for clients to establish there
				 * specific type and geometry and hence proceed to interact with them in an
				 * application specific manner.
				 * 
				 * Entities need to be able to indicate if they are still valid: even though
				 * they may have been part of a selection at a certain point in time, by the
				 * time that the entities of that selection are accessed the entities may not
				 * exist any more in reality.
				 */
				class Entity
				{

				public:
					/**
					 * Returns true if the given Entity corresponds to this Entity (i.e. the
					 * Tactical Objects are the same).
					 * 
					 * @return 
					 */
					virtual bool equals(std::auto_ptr<Entity> entity) =0;
					/**
					 * Returns the GeodeticPosition to be used as the main point of reference for this
					 * entity.
					 */
					virtual std::auto_ptr<org::omg::tacsit::geometry::GeodeticPosition> getReferencePoint() =0;
					/**
					 * Returns the type of the Entity. This corresponds to the application
					 * specific class of the tactical object that has been added to the TacSit.
					 * 
					 * @return 
					 */
					virtual std::auto_ptr<org::omg::tacsit::controller::EntityType> getType() =0;
					/**
					 * This is a convenience method for indicating whether the entity is to be
					 * handled as though it is a single point rather than a complex geometry.
					 * This is useful for containment and other queries.
					 * 
					 * @return 
					 */
					virtual bool isPointEntity() =0;
					/**
					 * Returns true if this instance is valid
					 * 
					 * @return 
					 */
					virtual bool isValid() =0;

				};

			}

		}

	}

}
#else
namespace org
{
	namespace omg
	{
		namespace tacsit
		{
			namespace controller
			{
				class Entity;
			}
		}
	}
}
#endif // !defined(EA_4AE38760_7BAE_4a2f_B9C7_AFCBB6C8A313__INCLUDED_)
