///////////////////////////////////////////////////////////
//  TacsitController.h
//  Implementation of the Interface TacsitController
//  Created on:      15-Mar-2012 09:41:29
///////////////////////////////////////////////////////////

#if !defined(EA_0C44550D_05DF_489d_A84B_7E5530C10EF8__INCLUDED_)
#define EA_0C44550D_05DF_489d_A84B_7E5530C10EF8__INCLUDED_

#include "Projection.h"
#include "QueryManager.h"
#include "SelectionManager.h"
#include "ViewportManager.h"
#include "SelectionMethodology.h"
#include "EntityType.h"

#include <string>
#include <memory>
#include <vector>
namespace org
{
	namespace omg
	{
		namespace tacsit
		{
			namespace controller
			{
				/**
				 * The top level interface for the Tactical Situation display (TacSit) Controller
				 * interface, which enables clients to interact with Viewports and Query TacSit
				 * content in terms of the Entity class.
				 * 
				 * The TacSit Controller interface does not provide the mechanism for adding
				 * Entities to a TacSit. When a client receives an Entity from a Query, the
				 * client of the TacSit controller will interpret the Entity according to
				 * interfaces provided by other components than the Tacsit Controller (for
				 * instance an interaction with another component, which is known to have added
				 * the Entity to the TacSit by some other interface).
				 */
				class TacsitController
				{

				public:
					/**
					 * Returns the projections which are supported by the TACSIT.
					 * 
					 * Attempted changes to the collection returned by this method will
					 * have no effect on the behavior and functionality of the TacsitController.
					 * 
					 * @return 
					 */
					virtual std::vector<std::auto_ptr<org::omg::tacsit::controller::Projection>> getProjections() =0;
					/**
					 * Return the QueryManager for this Tacsit Controller
					 * 
					 * @return 
					 */
					virtual std::auto_ptr<org::omg::tacsit::query::QueryManager> getQueryManager() =0;
					/**
					 * Return the SelectionManager that handles the selection on the viewport
					 * with name viewportName.
					 * 
					 * Depending on the SelectionMethodology of the Tacsit Controller the
					 * returned SelectionManagers for multiple viewports will be identical or
					 * different.
					 * 
					 * @return 
					 */
					virtual std::auto_ptr<org::omg::tacsit::controller::SelectionManager> getSelectionManager(std::string viewportName) =0;
					/**
					 * Return the ViewportManager for this Tacsit Controller.
					 * 
					 * @return 
					 */
					virtual std::auto_ptr<org::omg::tacsit::controller::ViewportManager> getViewportManager() =0;
					/**
					 * Returns the SelectionMethodology of the Tacsit controller
					 * 
					 * @return 
					 */
					virtual SelectionMethodology getSelectionMethodology() =0;
					/**
					 * Returns the Entity Types that are supported by the TAC SIT. This will return a
					 * 
					 * list of all Entity Types available for Selection and Query by this
					 * TACSITController.
					 * 
					 * Attempted changes to the collection returned by this method will have no effect
					 * 
					 * on the behavior and functionality of the TacsitController.
					 * @return 
					 */
					virtual std::vector<std::auto_ptr<org::omg::tacsit::controller::EntityType>> getEntityTypes() =0;

				};

			}

		}

	}

}
#else
namespace org
{
	namespace omg
	{
		namespace tacsit
		{
			namespace controller
			{
				class TacsitController;
			}
		}
	}
}
#endif // !defined(EA_0C44550D_05DF_489d_A84B_7E5530C10EF8__INCLUDED_)
