///////////////////////////////////////////////////////////
//  ViewEyeProperties.h
//  Implementation of the Class ViewEyeProperties
//  Created on:      15-Mar-2012 09:41:29
///////////////////////////////////////////////////////////

#if !defined(EA_95680D3D_2EA7_4a73_A635_A53738200244__INCLUDED_)
#define EA_95680D3D_2EA7_4a73_A635_A53738200244__INCLUDED_

#include "GeodeticPosition.h"
#include "Angle.h"
#include "Projection.h"
#include "Distance.h"

#include <string>
#include <memory>
#include <vector>
namespace org
{
	namespace omg
	{
		namespace tacsit
		{
			namespace controller
			{
				/**
				 * ViewEyeProperties groups the attributes of a ViewPort that have an impact on
				 * the position where Entities are displayed in the Viewport.
				 */
				class ViewEyeProperties
				{

				public:
					/**
					 * The GeoCenter of the Viewport.
					 * 
					 * The geoCenter of the Viewport is the Geoposition that is displayed in the
					 * center of the Viewport (i.e., on a Viewport of w pixels wide and h pixels
					 * high, it is displayed on pixel (w/2, h/2) ).
					 */
					virtual std::auto_ptr<org::omg::tacsit::geometry::GeodeticPosition> getGeoCenter() const =0;
					/**
					 * The orientation of the Viewport.
					 * 
					 * The orientation is applied on the Viewport as a rotation of the Viewport
					 * in clockwise direction around the Viewport's center. 
					 */
					virtual std::auto_ptr<org::omg::tacsit::geometry::Angle> getOrientation() const =0;
					/**
					 * The projection of the Viewport.
					 * 
					 * The projection object here is assumed to be one of the options provided
					 * by the high level TACSIT Controller method "getProjections()". This
					 * standard does not address any further details of how to handle
					 * projections.
					 */
					virtual std::auto_ptr<org::omg::tacsit::controller::Projection> getProjection() const =0;
					/**
					 * The rangescale of the Viewport.
					 * 
					 * The rangescale is the Distance from the center of the viewport to the
					 * nearest viewport edge.
					 */
					virtual std::auto_ptr<org::omg::tacsit::geometry::Distance> getRangeScale() const =0;
					/**
					 * The GeoCenter of the Viewport.
					 * 
					 * The geoCenter of the Viewport is the Geoposition that is displayed in the
					 * center of the Viewport (i.e., on a Viewport of w pixels wide and h pixels
					 * high, it is displayed on pixel (w/2, h/2) ).
					 */
					virtual void setGeoCenter(std::auto_ptr<org::omg::tacsit::geometry::GeodeticPosition> geoCenter) =0;
					/**
					 * The orientation of the Viewport.
					 * 
					 * The orientation is applied on the Viewport as a rotation of the Viewport
					 * in clockwise direction around the Viewport's center. 
					 */
					virtual void setOrientation(std::auto_ptr<org::omg::tacsit::geometry::Angle> orientation) =0;
					/**
					 * The projection of the Viewport.
					 * 
					 * The projection object here is assumed to be one of the options provided
					 * by the high level TACSIT Controller method "getProjections()". This
					 * standard does not address any further details of how to handle
					 * projections.
					 */
					virtual void setProjection(std::auto_ptr<org::omg::tacsit::controller::Projection> projection) =0;
					/**
					 * The rangescale of the Viewport.
					 * 
					 * The rangescale is the Distance from the center of the viewport to the
					 * nearest viewport edge.
					 */
					virtual void setRangeScale(std::auto_ptr<org::omg::tacsit::geometry::Distance> rangeScale) =0;

				};

			}

		}

	}

}
#else
namespace org
{
	namespace omg
	{
		namespace tacsit
		{
			namespace controller
			{
				class ViewEyeProperties;
			}
		}
	}
}
#endif // !defined(EA_95680D3D_2EA7_4a73_A635_A53738200244__INCLUDED_)
