///////////////////////////////////////////////////////////
//  Viewport.h
//  Implementation of the Interface Viewport
//  Created on:      15-Mar-2012 09:41:29
///////////////////////////////////////////////////////////

#if !defined(EA_DCC5609C_7330_4a46_A122_23557F6A5853__INCLUDED_)
#define EA_DCC5609C_7330_4a46_A122_23557F6A5853__INCLUDED_

#include "ViewportChangeListener.h"
#include "ScreenPosition.h"
#include "GeodeticPosition.h"
#include "ViewEyeProperties.h"
#include "Distance.h"

#include <string>
#include <memory>
#include <vector>
namespace org
{
	namespace omg
	{
		namespace tacsit
		{
			namespace controller
			{
				/**
				 * The Viewport class enables a TacSit client to interact with the basic
				 * properties of a TacSit view. In particular the view's name, where it is
				 * centered, how it is scaled, projected and oriented, and the selections it has
				 * (through its SelectionManager instance).
				 */
				class Viewport
				{

				public:
					/**
					 * Registers the given ViewportChangeListener to this Viewport.
					 * 
					 * After registration the ViewportChangeListener will be notified through
					 * its only method each time that a viewport is changed.
					 * 
					 * Registering a ViewportChangeListener that was already registered with
					 * this Viewport does not have any effect.
					 */
					virtual void addViewportChangeListener(std::auto_ptr<org::omg::tacsit::controller::ViewportChangeListener> listener) =0;
					/**
					 * Converts the given GeoPosition into a ScreenPosition (pixels). Note that
					 * the conversion may not be valid.
					 * 
					 * These conversion methods are useful if you are trying to do anything
					 * special with OS GUI libraries (e.g. mouse controllers, popup windows, or
					 * other things of that sort).
					 * 
					 * @return 
					 */
					virtual std::auto_ptr<org::omg::tacsit::geometry::ScreenPosition> convertGeoPosition(std::auto_ptr<org::omg::tacsit::geometry::GeodeticPosition> geoPos) =0;
					/**
					 * Converts the passed screen position into a geo position (pixels). Note
					 * that the conversion may not be valid.
					 * 
					 * These conversion methods are useful if you are trying to do anything
					 * special with OS GUI libraries (e.g. mouse controllers, popup windows, or
					 * other things of that sort).
					 * 
					 * @return 
					 */
					virtual std::auto_ptr<org::omg::tacsit::geometry::GeodeticPosition> convertScreenPosition(std::auto_ptr<org::omg::tacsit::geometry::ScreenPosition> screenPos) =0;
					/**
					 * Returns the name of the Viewport
					 * 
					 * @return 
					 */
					virtual std::string getName() =0;
					/**
					 * Returns a copy of the current ViewEyeProperties of the Viewport.
					 * Modification to the returned ViewEyeProperties object will have no effect
					 * on the Viewport. To effect change, call the setViewEye method with the
					 * desired properties.
					 * 
					 * @return 
					 */
					virtual std::auto_ptr<org::omg::tacsit::controller::ViewEyeProperties> getViewEye() =0;
					/**
					 * Unregisters the givenViewportChangeListener from this Viewport.
					 * 
					 * Afterwards the ViewportChangeListener will not be notified of Viewport
					 * changes.
					 * 
					 * Unregistering a ViewportChangeListener that was not registered does not
					 * have any effect.
					 */
					virtual void removeViewportChangeListener(std::auto_ptr<org::omg::tacsit::controller::ViewportChangeListener> listener) =0;
					/**
					 * Offset and scale the viewport to contain all points (as possible) passed
					 * in the given GeoPositions array. The margin Distance parameter specifies
					 * an additional space that needs to be visible around the broadest points
					 * in the points list.
					 */
					virtual void scaleToPoints(std::vector<std::auto_ptr<org::omg::tacsit::geometry::GeodeticPosition>> points, std::auto_ptr<org::omg::tacsit::geometry::Distance> margin) =0;
					/**
					 * Set the name of the Viewport to the given name
					 */
					virtual void setName(std::string name) =0;
					/**
					 * Set the ViewEye properties of the Viewport to the given
					 * ViewEyeProperties.
					 * 
					 * If any of the attributes of the given ViewEyeProperties object are not
					 * set, the current value of the Viewport for that attribute will not be
					 * affected.
					 * 
					 * Later changes to the given ViewEyeProperties object do not have an effect
					 * on the Viewport.
					 */
					virtual void setViewEye(std::auto_ptr<org::omg::tacsit::controller::ViewEyeProperties> viewEyeProps) =0;

				};

			}

		}

	}

}
#else
namespace org
{
	namespace omg
	{
		namespace tacsit
		{
			namespace controller
			{
				class Viewport;
			}
		}
	}
}
#endif // !defined(EA_DCC5609C_7330_4a46_A122_23557F6A5853__INCLUDED_)
