///////////////////////////////////////////////////////////
//  ViewportManager.h
//  Implementation of the Interface ViewportManager
//  Created on:      15-Mar-2012 09:41:30
///////////////////////////////////////////////////////////

#if !defined(EA_BE70692E_8C5F_4aaf_8F94_000A17C13E44__INCLUDED_)
#define EA_BE70692E_8C5F_4aaf_8F94_000A17C13E44__INCLUDED_

#include "Viewport.h"
#include "ViewportManagerListener.h"

#include <string>
#include <memory>
#include <vector>
namespace org
{
	namespace omg
	{
		namespace tacsit
		{
			namespace controller
			{
				/**
				 * The ViewportManager enables a TacSit client to manage Viewport instances and
				 * track changes to the set of Viewports. Construction of a Viewport is outside
				 * of the scope of this interface.
				 */
				class ViewportManager
				{

				public:
					/**
					 * Add a Viewport to this ViewportManager.
					 * 
					 * If the Viewport is already added to the ViewportManager before, this
					 * operation has no effect.
					 */
					virtual void addViewport(std::auto_ptr<org::omg::tacsit::controller::Viewport> viewport) =0;
					/**
					 * Registers the given ViewportManagerListener to this ViewportManager.
					 * 
					 * After registration the ViewportManagerListener will be notified through
					 * each time that a viewport is added or removed through its corresponding
					 * method.
					 * 
					 * Registering a ViewportManagerListener that was already registered with
					 * this ViewportManager does not have any effect.
					 */
					virtual void addViewportManagerListener(std::auto_ptr<org::omg::tacsit::controller::ViewportManagerListener> listener) =0;
					/**
					 * Return the viewport with the given name.
					 * 
					 * If there is no such viewport, null is returned.
					 * 
					 * @return 
					 */
					virtual std::auto_ptr<org::omg::tacsit::controller::Viewport> getViewport(std::string name) =0;
					/**
					 * Remove the given Viewport from this ViewportManager.
					 * 
					 * If the given Viewport is not managed by this ViewportManager, this
					 * operation as no effect.
					 */
					virtual void removeViewport(std::auto_ptr<org::omg::tacsit::controller::Viewport> viewport) =0;
					/**
					 * Unregisters the givenViewportManagerListener from this ViewportManager.
					 * 
					 * Afterwards the ViewportManagerListener will not be notified of added or
					 * removed Viewports.
					 * 
					 * Unregistering a ViewportManagerListener that was not registered does not
					 * have any effect.
					 */
					virtual void removeViewportManagerListener(std::auto_ptr<org::omg::tacsit::controller::ViewportManagerListener> listener) =0;
					/**
					 * Return all Viewports managed by this ViewportManager
					 * 
					 * The collection returned by this method is unmodifiable.
					 * @return 
					 */
					virtual std::vector<std::auto_ptr<org::omg::tacsit::controller::Viewport>> getViewports() =0;

				};

			}

		}

	}

}
#else
namespace org
{
	namespace omg
	{
		namespace tacsit
		{
			namespace controller
			{
				class ViewportManager;
			}
		}
	}
}
#endif // !defined(EA_BE70692E_8C5F_4aaf_8F94_000A17C13E44__INCLUDED_)
